#!/usr/bin/env python
# -*- coding: utf-8 -*-
#

# file      : wind_main.py
# project   : basebuildpy
# copyright : (C) 2014 by Claude BULIN
#
# This file is part of basebuildpy project
#
# This library is free software; you can redistribute it and/or
# modify it under the terms of the GNU Lesser General Public
# License as published by the Free Software Foundation; version 3.
#
# This library is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
# Lesser General Public License for more details.
#
# You should have received a copy of the GNU Lesser General Public
# License along with this library; if not, write to the Free Software
# Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA 02111-1307
# USA


import sys, os
try:
	from gi.repository import Gtk, GObject, Gdk
except:
	print("GTK Not Availible")
	sys.exit(1)
import subprocess, threading
from fconfig import FConfig
import Language_Init
from wind_about import wind_about
from prefs import Preferences
from wind_languages import wind_languages, wind_languages_init_list
from code_gen import codegen_py
from build_pot_file import build_pot_file
from wind_info import wind_info


# constant variables for notebook
(NOTEBOOK_GLADE, NOTEBOOK_TRANSLATE, NOTEBOOK_GENERATE) = range(3)


class wind_main():
	"""

	Management of the generation of a skeleton in python code

	"""
	def __init__( self, path="wind_main.glade" ):

		# Add language support
		_ = Language_Init.Translator(FConfig).lang.gettext
		self._ = _

		try:
			self.builder = Gtk.Builder()
			# Add language support from glade file
			self.builder.set_translation_domain("basebuildpy")
			self.builder.add_from_file( FConfig.DATADIR + "/" + path )
		except:
			print("except from:Gtk.Builder() ")
			sys.exit(1)

		self.ListNameFilesGlade = []
		self.ListPotFile = []

		self.wind_main=self.builder.get_object("wind_main")

		dic={
			"on_wind_main_delete_event": self.on_wind_main_delete_event,
			"on_wind_main_destroy_event": self.on_wind_main_destroy_event,
			"on_wind_main_key_press_event": self.on_wind_main_key_press_event,
			"on_wind_main_key_release_event": self.on_wind_main_key_release_event,
			"on_imagemenuitem_quit_activate": self.on_imagemenuitem_quit_activate,
			"on_menuitem_help_activate": self.on_menuitem_help_activate,
			"on_entry_project_name_changed": self.on_entry_project_name_changed,
			"on_entry_project_version_changed": self.on_entry_project_version_changed,
			"on_entry_project_author_changed": self.on_entry_project_author_changed,
			"on_button_project_destination_clicked": self.on_button_project_destination_clicked,
			"on_notebook_choice_switch_page": self.on_notebook_choice_switch_page,
			"on_label_drag_and_drop_realize": self.on_label_drag_and_drop_realize,
			"on_button_clear_glade_file_clicked": self.on_button_clear_glade_file_clicked,
			"on_button_add_glade_file_clicked": self.on_button_add_glade_file_clicked,
			"on_button_add_languages_clicked": self.on_button_add_languages_clicked,
			"on_button_generate_clicked": self.on_button_generate_clicked,
			"on_button_previous_clicked": self.on_button_previous_clicked,
			"on_button_next_clicked": self.on_button_next_clicked,
		}
		self.builder.connect_signals(dic)

		self.label_title_wind_main=self.builder.get_object("label_title_wind_main")
		self.entry_project_name=self.builder.get_object("entry_project_name")
		self.entry_project_version=self.builder.get_object("entry_project_version")
		self.entry_project_author=self.builder.get_object("entry_project_author")
		self.label_drag_and_drop=self.builder.get_object("label_drag_and_drop")
		self.label_see_languages=self.builder.get_object("label_see_languages")
		self.label_see_languages_1=self.builder.get_object("label_see_languages_1")
		self.button_generate=self.builder.get_object("button_generate")
		self.button_clear_glade_file=self.builder.get_object("button_clear_glade_file")
		self.button_project_destination=self.builder.get_object("button_project_destination")
		self.notebook_choice=self.builder.get_object("notebook_choice")
		self.button_previous=self.builder.get_object("button_previous")
		self.button_next=self.builder.get_object("button_next")
		self.box_previous_next=self.builder.get_object("box_previous_next")
		self.box_progressbar=self.builder.get_object("box_progressbar")
		self.progressbar_main=self.builder.get_object("progressbar_main")
		self.label_progressbar=self.builder.get_object("label_progressbar")
		self.timeout_id =  -1

		self.box_progressbar.hide()
		self.bool_error_make_msgfmt = False

		# Preferences: init
		self.Prefs = Preferences()
		self.Prefs.Preferences_read_file()
		# Preferences: set window position
		win_width      = self.Prefs.Preferences_get_win_width()
		win_height     = self.Prefs.Preferences_get_win_height()
		main_win_pos_x = self.Prefs.Preferences_get_main_win_pos_x()
		main_win_pos_y = self.Prefs.Preferences_get_main_win_pos_y()
		self.wind_main.resize( win_width, win_height )
		self.wind_main.move( main_win_pos_x, main_win_pos_y )
		# Preferences:
		self.entry_project_name.set_text( self.Prefs.Preferences_get_ProjectName() )
		self.entry_project_version.set_text( self.Prefs.Preferences_get_ProjectVersion() )
		self.entry_project_author.set_text( self.Prefs.Preferences_get_ProjectAuthor() )
		self.button_project_destination.set_label( self.Prefs.Preferences_get_form_ProjectDestination() )

		self.button_generate.set_sensitive(False)
		self.button_clear_glade_file.set_sensitive(False)

		# Default languages
		_ListUpdateLangs = self.Prefs.Preferences_get_LANGUAGES()
		WindLang = wind_languages_init_list()
		self.ListUpdateLangs = WindLang.init_list( _ListUpdateLangs )
		self.Update_languages( self.ListUpdateLangs )

		# init notebook in reverse
		list_page_notebook = [ 2, 1, 0 ]
		for page_notebook in list_page_notebook:
			self.notebook_choice.set_current_page( page_notebook )
		self.config_buttons_actions()

		# header window
# 		title_string = _("Generating a skeleton python from \none or more glade files")
# 		string = "<span font_desc=\"sans bold 12 \"><b>\n\n%s\n</b></span>" % title_string
# 		self.label_title_wind_main.set_markup( string )

		# header
		title_string = "Base Build Py"
		comment_string = "Generating a skeleton python from one or more glade files"
		string = "<span font_desc=\"sans bold 20 \">\n<b>%s</b>\n</span><b>%s</b>" % (title_string, comment_string)
		self.label_title_wind_main.set_markup( string )

		self.wind_main.set_title("%s - Version %s" % (FConfig.__PACKAGE__, FConfig.__VERSION__))
		self.wind_main.set_icon_from_file( FConfig.DATADIR + "/basebuildpy.png" )
		self.wind_main.show()


	#
	# End of: Bye
	#
	def main_save_preferences(self):
		"""
		Required:
			Nothing

		Returns:
			Nothing

		Logic:
			Save options before quitting
		"""
		x, y = self.wind_main.get_position()
		width, height = self.wind_main.get_size()
		self.Prefs.Preferences_save_op(
			self.entry_project_name.get_text(),
			self.entry_project_version.get_text(),
			self.entry_project_author.get_text(),
			self.button_project_destination.get_label(),
			self.ListUpdateLangs,
			x, y, width, height)

	def on_wind_main_delete_event(self, widget, data=None):
		self.main_save_preferences()
		Gtk.main_quit()

	def on_wind_main_destroy_event(self, widget, data=None):
		self.main_save_preferences()
		Gtk.main_quit()

	def on_imagemenuitem_quit_activate(self, widget, data=None):
		self.main_save_preferences()
		Gtk.main_quit()


	#
	# Key press/release event
	#
	def on_wind_main_key_press_event(self, widget, data=None):
		pass
	def on_wind_main_key_release_event(self, widget, data=None):
		pass


	#
	# About
	#
	def on_menuitem_help_activate(self, widget, data=None):
		"""
		Required:
			Nothing

		Returns:
			Nothing

		Logic:
			Call wind_about()
		"""
		wind_about(Form=self.wind_main)


	#
	# Updating buttons
	#
	def config_buttons_actions(self):
		"""
		Required:
			Nothing

		Returns:
			Nothing

		Logic:
			Updating buttons
		"""
		self.button_generate.set_sensitive( False )
		self.button_clear_glade_file.set_sensitive( False )
		self.button_previous.set_sensitive( False )
		self.button_next.set_sensitive( False )

		txt_name_prj = self.entry_project_name.get_text().strip()
		txt_version_prj = self.entry_project_version.get_text().strip()
		txt_author_prj = self.entry_project_author.get_text().strip()
		txt_drag = self.label_drag_and_drop.get_label().strip()
		txt_lang = self.label_see_languages.get_label().strip()

		path_name_file = self.button_project_destination.get_label() + "/" + txt_name_prj + "-" + txt_version_prj
		self.builder.get_object("label_name_dest_project").set_markup( "<span color=\"green\"><b>%s</b></span>" % path_name_file )

		new_page = self.notebook_choice.get_current_page()
		if new_page == NOTEBOOK_GLADE:
			if txt_drag != "":
				self.button_clear_glade_file.set_sensitive( True )
			if txt_name_prj != "" and txt_version_prj != "" and txt_author_prj != "" and txt_drag != "":
				idem_name = False
				for name_file in self.ListNameFilesGlade:
					BaseName = os.path.basename( name_file )
					Name = BaseName.split(".")[0]
					Ext  = BaseName.split(".")[-1]
					if Name == self.entry_project_name.get_text():
						idem_name = True
						break
				if idem_name == False:
					self.button_next.set_sensitive( True )

		if new_page == NOTEBOOK_TRANSLATE:
			self.button_previous.set_sensitive( True )
			if txt_name_prj != "" and txt_version_prj != "" and txt_author_prj != "" and txt_lang != "":
				self.button_next.set_sensitive( True )

		if new_page == NOTEBOOK_GENERATE:
			self.button_previous.set_sensitive( True )
			if txt_name_prj != "" and txt_version_prj != "" and txt_author_prj != "":
				self.button_generate.set_sensitive( True )


	#
	# Buttons init with notebook_switch_page
	#
	def on_notebook_choice_switch_page(self, obj1, obj2, page):
		"""
		Required:
			page -- Page numerate of notebook

		Returns:
			Nothing

		Logic:
			Active ou désactive les boutons next-previous
			call: config_buttons_actions()
		"""
		if page == NOTEBOOK_GLADE:
			self.button_previous.hide()
			self.button_next.show()
		elif page == NOTEBOOK_GENERATE:
			self.button_previous.show()
			self.button_next.hide()
		else:
			self.button_previous.show()
			self.button_next.show()
		self.config_buttons_actions()

	def on_button_previous_clicked(self, widget, data=None):
		self.notebook_choice.prev_page()
		self.config_buttons_actions()

	def on_button_next_clicked(self, widget, data=None):
		self.notebook_choice.next_page()
		self.config_buttons_actions()


	#
	# List from 'Drag and drop' and FileChooserDialog
	#
	def analyse_list_glade_file( self, p_list_glade_file ):
		"""
		Required:
			p_list_glade_file -- a new list of glade file

		Returns:
			Nothing

		Logic:
			Display file names glade
			File names can not be identical
			The project name must be different from the file name glade
		"""
		bool_import_error = False
		PathDest = self.button_project_destination.get_label() + "/" + self.entry_project_name.get_text() + "-" + self.entry_project_version.get_text()
		for name_file in p_list_glade_file:
			if os.path.isfile( name_file ):
				BaseName = os.path.basename( name_file )
				Name = BaseName.split(".")[0]
				Ext  = BaseName.split(".")[-1]
				# You can not import a GLADE file from the destination folder !
				if PathDest in name_file:
					bool_import_error = True
				elif Ext.lower() == 'glade':
					# test if 'name_file' in liste
					bool_equal = False
					for item in self.ListNameFilesGlade:
						if item == name_file:
							bool_equal = True
							break
					# if ok
					if bool_equal == False:
						# File names can not be identical
						for item in self.ListNameFilesGlade:
							Item_BaseName = os.path.basename( item )
							Item_Name = Item_BaseName.split(".")[0]
							Item_Ext  = Item_BaseName.split(".")[-1]
							if Item_Name == Name:
								bool_equal = True
								break
						# if ok
						if bool_equal == False:
							txt = self.label_drag_and_drop.get_label()
							if Name == self.entry_project_name.get_text():
								Name = "<b>***  " + Name + "</b>"
							self.label_drag_and_drop.set_markup( txt + "\n" + Name + "." + Ext )
							# add pathname_file in liste
							self.ListNameFilesGlade.append( name_file )
		self.config_buttons_actions()
		if bool_import_error == True:
				wind_info(
					Form=self.wind_main,
					Title=_("Import error"),
					Text=_("You can not import a GLADE file from the destination folder !"))
		# The project name must be different from the file name glade
		for name_file in self.ListNameFilesGlade:
			BaseName = os.path.basename( name_file )
			Name = BaseName.split(".")[0]
			Ext  = BaseName.split(".")[-1]
			if Name == self.entry_project_name.get_text():
				wind_info(
					Form=self.wind_main,
					Title=_("Warning name project"),
					Text=_("The project name must be different from the file name glade !"))
				break



	#
	# Drag and drop
	#
	def on_drag_data_received( self, widget, drag_context, x,y, data,info, time):
		"""
		Required:
			data -- list of import files

		Returns:
			Nothing

		Logic:
			Stores the file names in a list
			call analyse_list_glade_file()
		"""
		p_list_glade_file = []
		for uri in data.get_uris():
			# Example uri: file:///home/cat/DEV/BASE_PYTHON/wind_main.glade
			name_file = uri[ 7: ]
			p_list_glade_file.append( name_file )
		self.analyse_list_glade_file( p_list_glade_file )


	#
	# Realize for drag and drop
	#
	def on_label_drag_and_drop_realize(self, widget, data=None):
		"""
		Required:
			widget -- adress of widget

		Returns:
			Nothing

		Logic:
			Realize for drag and drop
		"""
		self.AdrDrawingarea = widget
		widget.drag_dest_set(
			Gtk.DestDefaults.MOTION |
			Gtk.DestDefaults.HIGHLIGHT |
			Gtk.DestDefaults.DROP,
			[Gtk.TargetEntry.new("text/uri-list", 0, 80)],
			Gdk.DragAction.COPY,
			)
		widget.connect("drag-data-received", self.on_drag_data_received)


	#
	# Clear list glade file
	#
	def on_button_clear_glade_file_clicked(self, widget, data=None):
		self.ListNameFilesGlade[:] = []
		self.label_drag_and_drop.set_label( "" )
		self.config_buttons_actions()


	#
	# Add list glade file
	#
	def on_button_add_glade_file_clicked(self, widget, data=None):
		dialog = Gtk.FileChooserDialog(
			_("Please choose a glade file"),
			None,
			Gtk.FileChooserAction.OPEN,
			(Gtk.STOCK_CANCEL, Gtk.ResponseType.CANCEL,
			_("Select"),       Gtk.ResponseType.OK))
		dialog.set_current_folder( self.Prefs.Preferences_get_home() )
		dialog.set_default_size(400, 400)
		dialog.set_select_multiple( True )
		response = dialog.run()
		if response == Gtk.ResponseType.OK:
			self.analyse_list_glade_file( dialog.get_filenames() )
		dialog.destroy()


	#
	# Entry changed
	#
	def on_entry_project_name_changed(self, widget, data=None):
		# The project name must be different from the file name glade
		self.label_drag_and_drop.set_label( "" )
		idem_name = False
		for name_file in self.ListNameFilesGlade:
			BaseName = os.path.basename( name_file )
			Name = BaseName.split(".")[0]
			Ext  = BaseName.split(".")[-1]
			if Name == widget.get_text():
				Name = "<b>***  " + Name + "</b>"
				idem_name = True
			self.label_drag_and_drop.set_markup( self.label_drag_and_drop.get_label() + "\n" + Name + "." + Ext )
		if idem_name == True:
			wind_info(
				Form=self.wind_main,
				Title=_("Warning name project"),
				Text=_("The project name must be different from the file name glade !"))
		self.config_buttons_actions()



	def on_entry_project_version_changed(self, widget, data=None):
		self.config_buttons_actions()

	def on_entry_project_author_changed(self, widget, data=None):
		self.config_buttons_actions()


	#
	# Add languages and update list
	#
	def Update_languages(self, UpdateList=[]):
		label = ""
		label_1 = ""
		self.ListPotFile[:] = []
		_ListUpdateLangs = []
		for item in UpdateList:
			label = label + item[1] + "\n"
			label_1 = label_1 + item[2] + "\n"
			self.ListPotFile.append( item[2] + ".po" )
			_ListUpdateLangs.append( [ item[0], item[1], item[2] ] )
		# if "en.po" in self.ListPotFile:
		# 	self.ListPotFile.append( "en_GB.po" )
		self.ListUpdateLangs[:] = []
		self.ListUpdateLangs = _ListUpdateLangs
		label = label.strip()
		label_1 = label_1.strip()
		self.label_see_languages.set_label( label )
		self.label_see_languages_1.set_label( label_1 )
		self.config_buttons_actions()

	def on_button_add_languages_clicked(self, widget, data=None):
		wind_languages(Form=self.wind_main, FuncUpdate=self.Update_languages)


	#
	# Select folder to project destination
	#
	def folder_button_project_destination_clicked(self):
		dialog = Gtk.FileChooserDialog(
			_("Please choose a folder"),
			None,
			Gtk.FileChooserAction.SELECT_FOLDER,
			(Gtk.STOCK_CANCEL,
			Gtk.ResponseType.CANCEL,
			_("Select"),
			Gtk.ResponseType.OK))
		dialog.set_default_size(400, 400)
		response = dialog.run()
		if response == Gtk.ResponseType.OK:
			self.button_project_destination.set_label( dialog.get_filename() )
			self.config_buttons_actions()
		dialog.destroy()

	def on_button_project_destination_clicked(self, widget, data=None):
		self.folder_button_project_destination_clicked()


	#
	# Creation of source code and folders
	#
	def create_project(self, PathDest):
		print("\nTreatment: Creation of source code and related files")
		print("-------------------------------------------------------")
		codegen = codegen_py()
		codegen.create_template(
					self.entry_project_name.get_text(),
					self.entry_project_version.get_text(),
					self.entry_project_author.get_text(),
					self.button_project_destination.get_label(),
					self.ListNameFilesGlade
					)
		BuildPotFile = build_pot_file(
					self.wind_main,
					PathDest,
					self.ListPotFile,
					self.entry_project_name.get_text(),
					self.entry_project_version.get_text(),
					self.entry_project_author.get_text()
					)
		self.bool_error_make_msgfmt = BuildPotFile.create_pot_file()
		self.timeout_id = -1

	def timeout_create_project(self, user_data):
		self.progressbar_main.pulse()
		if self.timeout_id > 0:
			return True
		else:
			self.box_previous_next.show()
			self.box_progressbar.hide()
			if self.bool_error_make_msgfmt == True:
				wind_info(
					Form=self.wind_main,
					Title=_("Warning from msgfmt"),
					Text=_("Do not forget to correct errors in the text of PO files for MO files"))
			return False

	def thread_create_project(self, PathDest):
		self.box_previous_next.hide()
		self.box_progressbar.show()
		self.label_progressbar.set_label( _("Please wait while the treatment") )
		self.timeout_id = GObject.timeout_add( 100, self.timeout_create_project, None )
		thread = threading.Thread( target=self.create_project, args=(PathDest,))
		thread.start()

	def on_button_generate_clicked(self, widget, data=None):
		PathDest = self.button_project_destination.get_label() + "/" + self.entry_project_name.get_text() + "-" + self.entry_project_version.get_text()
		if os.path.isdir( PathDest ) == True:
			dialogWindow = Gtk.MessageDialog(None,
								  Gtk.DialogFlags.MODAL | Gtk.DialogFlags.DESTROY_WITH_PARENT,
								  Gtk.MessageType.QUESTION,
								  Gtk.ButtonsType.OK_CANCEL,
								  _("This folder exist.\nDo you want to replace this folder ?") )
			response = dialogWindow.run()
			dialogWindow.destroy()
			while Gtk.events_pending():
				Gtk.main_iteration()
			if response == Gtk.ResponseType.OK:
				# remove directory
				command = "rm -rf " + PathDest
				subprocess.call( command, shell=True )
				#
				self.thread_create_project( PathDest )
		else:
			self.thread_create_project( PathDest )


if __name__ == '__main__':
	wind = wind_main()
	Gtk.main()
